==> softgun_0.16.orig.tar.gz: softgun-0.16/shlib.c <==
/*
 * ----------------------------------------------------------
 *
 * Load Emulation modules as shared libraries
 * Intended to allow proprietary device emulators.
 * It is always recommended to publish source but sometimes
 * this is not possible because of non disclosure agreements 
 *
 * Special hate to the following Vendors:
 * 	Broadcom
 *	MSystems
 *
 * Usage: create a shared library libhello.so with an initfunction 
 *        void _init(void)
 *	  and add a libpath and a list of shared libraries to your 
 *	  emulator configuration file.
 *
 * (C) 2005  Lightmaze Solutions AG
 *   Author: Jochen Karrer
 *
 * State:
 *	Working	
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ----------------------------------------------------
 */

/* I do not know how other architectures handle shared libraries */
#ifdef __linux__
#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "configfile.h"

/*
 * --------------------------------------------------
 * Split a string at whitespace and write
 * to an argv vector. Warning: string is modified 
 * --------------------------------------------------
 */
static int
split_args(char *str,char *argv[],int maxargs) {
	int i;
	int inspace=1;
	int argc=0;
	for(i=0;*str;i++,str++) {
		if((*str!=' ') && (*str != '\t')) {
			if(inspace) {
				argv[argc]=str;
				argc++;
				if(argc>=maxargs) {
					fprintf(stderr,"Shared library list to long\n");
					exit(32424);
				}
			}
			inspace=0;
		} else {
			inspace=1;
			*str=0;
		}
	}
	return argc;
}
/*
 * ---------------------------------------
 * Split a path-string at ';' or ':'
 * Warning: string is modified !
 * ---------------------------------------
 */
static int
split_path(char *str,char *argv[],int maxargs) {
	int i;
	int insemicolon=1;
	int argc=0;
	for(i=0;*str;i++,str++) {
		if((*str==';')) {
			insemicolon=1;
			*str=0;
		} else if((*str==':')) {
			insemicolon=1;
			*str=0;
		} else {
			if(insemicolon) {
				argv[argc]=str;
				argc++;
				if(argc>=maxargs) {
					fprintf(stderr,"Path has to many components\n");
					exit(32424);
				}
			}
			insemicolon=0;
		}
	}
	return argc;
}
/*
 * ----------------------------------------------------
 * Load a shared library and call its init function
 * ----------------------------------------------------
 */
static int
Shlib_Load(const char *dirname,const char *basenm) 
{
	char *path = alloca(strlen(dirname)+strlen(basenm)+2);
	void *dlhandle;
	struct stat stat_buf;
	sprintf(path,"%s/%s",dirname,basenm);
	if(stat(path,&stat_buf)<0) {
		//fprintf(stderr,"Can't stat\"%s\", dirname \"%s\"\n",path,dirname);
		return -1;	
	}
	dlhandle = dlopen(path,RTLD_NOW | RTLD_GLOBAL);
	if(!dlhandle)  {
		fprintf (stderr,"dlopen error: %s\n",dlerror());
		exit(3508);
	}
	return 0;	
}

#define MAXSHLIBS (30)
#define MAXDIRS (30)

void
Shlibs_Init(void) {
	char *shlibs;
	char *libpath;
	char *pathdup;
	char *libs[MAXSHLIBS];
	char *dirs[MAXDIRS];
	int libc;
	int dirc;
	int i,j;
	shlibs = Config_ReadVar("global","libs");
	if(!shlibs)
		return;
	libc=split_args(shlibs,libs,MAXSHLIBS);
	libpath = Config_ReadVar("global","libpath");
	if(!libpath) {
		libpath=".";
	}
	pathdup = strdup(libpath);
	dirc=split_path(pathdup,dirs,MAXDIRS);
	for(i=0;i<libc;i++) {
		int retval=-2;
		for(j=0;j<dirc;j++) {
			retval = Shlib_Load(dirs[j],libs[i]); 
			if(retval>=0) {
				break;
			}	
		}
		if(retval<0) {
			fprintf(stderr,"Can not open lib \"%s\" in path \"%s\"\n",libs[i],libpath);
			exit(34223);
		}
	}
	free(pathdup);
}
#ifdef SHLIB_TEST
int 
main() {
	Shlib_Load("libbla.so");	
	Shlib_Load("laber.so.2.3");	
}
#endif
#endif

==> softgun_0.18.orig.tar.gz: softgun-0.18/shlib.c <==
/*
 * ----------------------------------------------------------
 *
 * Load Emulation modules as shared libraries
 * Intended to allow proprietary device emulators.
 * It is always recommended to publish source but sometimes
 * this is not possible because of non disclosure agreements 
 *
 * Special hate to the following Vendors:
 * 	Broadcom
 *	MSystems
 *
 * Usage: create a shared library libhello.so with an initfunction 
 *        void _init(void)
 *	  and add a libpath and a list of shared libraries to your 
 *	  emulator configuration file.
 *
 * (C) 2005  Lightmaze Solutions AG
 *   Author: Jochen Karrer
 *
 * State:
 *	Working	
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ----------------------------------------------------
 */

/* I do not know how other architectures handle shared libraries */

#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "configfile.h"
#include "sgstring.h"

/*
 * --------------------------------------------------
 * Split a string at whitespace and write
 * to an argv vector. Warning: string is modified 
 * --------------------------------------------------
 */
static int
split_args(char *str,char *argv[],int maxargs) {
	int i;
	int inspace=1;
	int argc=0;
	for(i=0;*str;i++,str++) {
		if((*str!=' ') && (*str != '\t')) {
			if(inspace) {
				argv[argc]=str;
				argc++;
				if(argc>=maxargs) {
					fprintf(stderr,"Shared library list to long\n");
					exit(32424);
				}
			}
			inspace=0;
		} else {
			inspace=1;
			*str=0;
		}
	}
	return argc;
}
/*
 * ---------------------------------------
 * Split a path-string at ';' or ':'
 * Warning: string is modified !
 * ---------------------------------------
 */
static int
split_path(char *str,char *argv[],int maxargs) {
	int i;
	int insemicolon=1;
	int argc=0;
	for(i=0;*str;i++,str++) {
		if((*str==';')) {
			insemicolon=1;
			*str=0;
		} else if((*str==':')) {
			insemicolon=1;
			*str=0;
		} else {
			if(insemicolon) {
				argv[argc]=str;
				argc++;
				if(argc>=maxargs) {
					fprintf(stderr,"Path has to many components\n");
					exit(32424);
				}
			}
			insemicolon=0;
		}
	}
	return argc;
}
/*
 * ----------------------------------------------------
 * Load a shared library and call its init function
 * ----------------------------------------------------
 */
static int
Shlib_Load(const char *dirname,const char *basenm) 
{
	char *path = alloca(strlen(dirname)+strlen(basenm)+2);
	void *dlhandle;
	struct stat stat_buf;
	sprintf(path,"%s/%s",dirname,basenm);
	if(stat(path,&stat_buf)<0) {
		//fprintf(stderr,"Can't stat\"%s\", dirname \"%s\"\n",path,dirname);
		return -1;	
	}
	dlhandle = dlopen(path,RTLD_NOW | RTLD_GLOBAL);
	if(!dlhandle)  {
		fprintf (stderr,"dlopen error: %s\n",dlerror());
		exit(3508);
	}
	return 0;	
}

#define MAXSHLIBS (30)
#define MAXDIRS (30)

void
Shlibs_Init(void) {
	char *shlibs;
	char *libpath;
	char *pathdup;
	char *libs[MAXSHLIBS];
	char *dirs[MAXDIRS];
	int libc;
	int dirc;
	int i,j;
	shlibs = Config_ReadVar("global","libs");
	if(!shlibs)
		return;
	libc=split_args(shlibs,libs,MAXSHLIBS);
	libpath = Config_ReadVar("global","libpath");
	if(!libpath) {
		libpath=".";
	}
	pathdup = sg_strdup(libpath);
	dirc=split_path(pathdup,dirs,MAXDIRS);
	for(i=0;i<libc;i++) {
		int retval=-2;
		for(j=0;j<dirc;j++) {
			retval = Shlib_Load(dirs[j],libs[i]); 
			if(retval>=0) {
				break;
			}	
		}
		if(retval<0) {
			fprintf(stderr,"Can not open lib \"%s\" in path \"%s\"\n",libs[i],libpath);
			exit(34223);
		}
	}
	free(pathdup);
}
#ifdef SHLIB_TEST
int 
main() {
	Shlib_Load("libbla.so");	
	Shlib_Load("laber.so.2.3");	
}
#endif
